from argparse import ArgumentParser

from pycreator.actions.add.add_action import AddAction
from pycreator.actions.create.create_action import CreateAction
from pycreator.actions.goes.goes_action import GoesAction
from pycreator.actions.version import ShowVersion
from pycreator.framework.messages import Messages


class ActionDispatcher:
    ACTION_HANDLERS = [AddAction,
                       CreateAction,
                       GoesAction,
                       ShowVersion]

    def __init__(self):
        self.parser = ArgumentParser()
        subparsers = self.parser.add_subparsers()
        self.action_handlers = {action_handler.ACTION: action_handler(subparsers) for action_handler in
                                self.ACTION_HANDLERS}

    def process_application(self):
        configuration = self.parser.parse_args()
        try:
            self.action_handlers[configuration.ACTION].process_action(configuration)
        except AttributeError:
            Messages.clean("pycreator")
            Messages.clean("quick tool to create a Python cmd boilerplate application\n")
            Messages.clean("Choose operation:"
                           "\n\t\t - create -- for creation a new package"
                           "\n\t\t - goes   -- for brrrr")
            Messages.clean("")



from typing import Tuple, Dict

from pycreator.framework.messages import Messages


def get_internal_paths(app_name):
    return (
        ("src", app_name),
        ("src", app_name, "actions"),
        ("src", app_name, "actions", "example_action"),
        ("src", app_name, "core"),
        ("src", app_name, "framework"),
        ("src", app_name, "main")
    )


def create_application_dirs(location, app_name: str):
    import os
    internal_paths = (
        ("src",),
        *get_internal_paths(app_name)
    )
    # TODO add verification before this method
    os.makedirs(os.path.join(location, app_name))

    for internal_path in internal_paths:
        os.makedirs(os.path.join(*(location, app_name, *internal_path)))


def create_inits(location, app_name: str):
    import os
    internal_paths = get_internal_paths(app_name)

    for internal_path in internal_paths:
        # if not os.path.exists(os.path.join(location, internal_path)):
        #     continue
        with open(os.path.join(*(location, app_name, *internal_path, "__init__.py")), 'w') as init_file:
            init_file.write('')


def create_additional_files(location, app_name: str):
    import os
    internal_paths = (
        "README.md",
        "VERSION",
        "LICENSE.md"
    )

    for internal_path in internal_paths:
        with open(os.path.join(*(location, app_name, internal_path)), 'w') as init_file:
            init_file.write(f"{app_name} {internal_path}")


def create_pycreator_hook(location):
    """
    Create a '.pycreator' file which works like a lock for the pycreator app.
    :param location: Absolute path to the root directory of the new application
    :return: None
    """
    import os
    with open(os.path.join(location, '.pycreator')) as pycreator_hook:
        pycreator_hook.write('')


class Template:
    def __init__(self, location: Tuple, dest_name: str, template_name: str, render_vars: Dict = None) -> None:
        self.location = location
        self.dest_name = dest_name
        self.template_name = template_name
        self.render_vars = render_vars if render_vars else {}


def create_files_from_templates(location: str, app_name: str):
    import os
    from jinja2 import Environment, PackageLoader, select_autoescape
    env = Environment(
        loader=PackageLoader('pycreator', 'templates'),
        autoescape=select_autoescape(['html', 'xml', 'j2'])
    )

    files_per_template = (
        Template(("src", app_name), 'setup.py', 'setup.j2', {"creator_app_name": app_name}),
        Template(("src", app_name, "main"), "main.py", "main.j2", {"creator_app_name": app_name}),
        Template(("src", app_name, "framework"), "messages.py", "messages.j2", {"creator_app_name": app_name}),
        Template(("src", app_name, "actions"), "version.py", "version.j2", {"creator_app_name": app_name}),
        Template(("src", app_name, "actions"), "action_dispatcher.py", "action_dispatcher.j2",
                 {"creator_app_name": app_name}),
        Template(("src", app_name, "actions"), "action.py", "action.j2", {"creator_app_name": app_name}),
        Template(("src", app_name, "actions", "example_action"), "example_action.py", "example_action.j2",
                 {"creator_app_name": app_name,
                  "creator_class_name": "ExampleAction",
                  "creator_class_name_action": "example"}),
        Template(("src", app_name), ".pycreator.lock", ".pycreator.j2", {}),
    )

    for file_template in files_per_template:
        path_file_template = os.path.join(*(location, app_name, *file_template.location, file_template.dest_name))
        with open(path_file_template, 'w') as new_file:
            template = env.get_template(file_template.template_name)
            new_file.write(template.render(**file_template.render_vars))


def create_new_action(location_to_actions_dir: str, app_name: str):
    import os
    from jinja2 import Environment, PackageLoader, select_autoescape
    env = Environment(
        loader=PackageLoader('pycreator', 'templates'),
        autoescape=select_autoescape(['html', 'xml', 'j2'])
    )

    file_template = Template((location_to_actions_dir, app_name), f"{app_name}.py", "action.j2",
                             {"creator_app_name": app_name,
                              "creator_class_name": f"{app_name[0].upper()}{app_name[1:]}Action",
                              "creator_class_name_action": app_name})

    if os.path.exists(os.path.join(*(location_to_actions_dir, app_name, *file_template.location))):
        Messages.error(f"Can't create a new action. Action {app_name} exists")
        return False
    os.makedirs(os.path.join(*(location_to_actions_dir, app_name, *file_template.location)))

    path_file_template = os.path.join(
        *(location_to_actions_dir, app_name, *file_template.location, file_template.dest_name))
    with open(path_file_template, 'w') as new_file:
        template = env.get_template(file_template.template_name)
        new_file.write(template.render(**file_template.render_vars))
    return True



import os

from pycreator.actions.action import Action
from pycreator.core import create_folders
from pycreator.framework.messages import Messages


class CreateAction(Action):
    ACTION = "create"
    PARAM_NAME = "ACTION"

    def fill_parser_arguments(self):
        self.parser.add_argument('-n', '--name', type=str, help='Name for the new project', required=True)
        self.parser.add_argument('-l', '--location', type=str, help='Custom location to create a new app', default=None)

    def process_action(self, configuration):
        target_location = configuration.location if configuration.location else os.getcwd()
        if self.check_app_exists(target_location, configuration.name):
            exit(1)
        if not self.create_boilerplate(target_location, configuration.name):
            Messages.error(f"Some error occur. The pycreator does not create an application {configuration.name}"
                           f" at {target_location}")
            return
        Messages.ok(f"The pycreator successfully create a new application {configuration.name}"
                    f" at {target_location}")

    @staticmethod
    def create_boilerplate(tgt_location: str, app_name: str):
        try:
            initial_steps = (
                create_folders.create_application_dirs,
                create_folders.create_inits,
                create_folders.create_additional_files,
                create_folders.create_files_from_templates
            )
            for step in initial_steps:
                step(tgt_location, app_name)
        except Exception as e:
            print(e)
            return False
        return True

    @staticmethod
    def check_app_exists(tgt_location: str, app_name: str):
        potential_app_location = os.path.join(tgt_location, app_name)
        if os.path.exists(potential_app_location) and os.path.isdir(potential_app_location):
            Messages.error(f"Application {app_name} exist at {tgt_location}. Choose different name or location.")
            return True
        return False



import os

from pycreator.actions.action import Action
from pycreator.core.create_folders import create_new_action
from pycreator.framework.messages import Messages


class AddAction(Action):
    ACTION = "add"
    PARAM_NAME = "ACTION"

    def fill_parser_arguments(self):
        self.parser.add_argument('-n', '--name', type=str, help="Create new application")

    def process_action(self, configuration):
        self._is_correct_cwd(new_action_name=configuration.name)

    def _is_correct_cwd(self, new_action_name: str) -> bool:
        if not self._available_to_create_directory("actions"):
            return False
        action_location = os.path.join(os.getcwd(), "actions")
        if not create_new_action(action_location,  new_action_name):
            return False

        if not os.path.exists(os.path.join(action_location, new_action_name)):
            Messages.warn(f"{new_action_name} action dir was not created")
            return False
        Messages.info(f"New action {new_action_name} was created")
        Messages.info(f"Remember to import a '{new_action_name}' at 'actions_dispatcher.py'")
        return True

    @staticmethod
    def _available_to_create_directory(dir_name) -> bool:
        # try to find a "actions" directory
        working_dir = os.getcwd()
        if os.path.exists(os.path.join(working_dir, dir_name)):
            return True
        return False



from pycreator.actions.create.create_action import CreateAction


class GoesAction(CreateAction):
    """
    This is a dummy action only to create a link for 'goes' shortcut for the 'create' option.
    """
    ACTION = "goes"

    def fill_parser_arguments(self):
        self.parser.add_argument('name', type=str, help='Name for the new project')
        self.parser.add_argument('-l', '-at', '--location', type=str, help='Custom location to create a new app')
